info = {
    'name': 'CrossingTraffic_POMDP',
    'description': 'In a grid, a robot must get to a goal and avoid obstacles arriving randomly and moving left. This is the pomdp version',
    'context': 'ippc2014',
    'tags': '',
    'viz': 'None'
}


info = {
    'name': 'Elevators',
    'description': 'The Elevator domain models evening rush hours when people from different floors in a building want to go down to the bottom floor using elevators.',
    'context': '',
    'tags': '',
    'viz': 'ElevatorViz.ElevatorVisualizer'
}


info = {
    'name': 'RedFinnedBlueEye',
    'description': 'The Red-finned Blue-eye domain tackles the problem of eradicating the invasive Gambusia from the habitat of the red-finned blue-eye.',
    'context': 'ippc2018',
    'tags': '',
    'viz': 'None'
}



info = {
    'name': 'Manufacturer',
    'description': 'In this domain, the agent manages a manufacturing company that buys goods to use them in the production of other goods.',
    'context': 'ippc2018',
    'tags': '',
    'viz': 'None'
}





info = {
    'name': 'MarsRover',
    'description': 'Multi Rover Navigation, where a group of agent needs to harvest mineral.',
    'context': '',
    'tags': '',
    'viz': 'MarsRoverViz.MarsRoverVisualizer'
}


import matplotlib.pyplot as plt
import numpy as np
from PIL import Image

from pyRDDLGym.Core.Compiler.RDDLModel import PlanningModel
from pyRDDLGym import Visualizer
from pyRDDLGym.Visualizer.StateViz import StateViz


class MarsRoverVisualizer(StateViz):

    def __init__(self, model: PlanningModel,
                 figure_size=[50, 50],
                 dpi=20,
                 fontsize=8,
                 display=False) -> None:
        self._model = model
        self._states = model.groundstates()
        self._nonfluents = model.groundnonfluents()
        self._objects = model.objects
        self._figure_size = figure_size
        self._dpi = dpi
        self._fontsize = fontsize
        self._interval = 10
        self._asset_path = "/".join(Visualizer.__file__.split("/")[:-1])
        self._nonfluent_layout = None
        self._state_layout = None
        self._fig, self._ax = None, None
        self._data = None
        self._img = None

    def build_nonfluents_layout(self):
        mineral_locaiton = {o: [None, None, None, None]
                            for o in self._objects['mineral']}

        # style of fluent_p1
        for k, v in self._nonfluents.items():
            var, objects = self._model.parse(k)
            if var == 'MINERAL-POS-X':
                mineral_locaiton[objects[0]][0] = v
            elif var == 'MINERAL-POS-Y':
                mineral_locaiton[objects[0]][1] = v
            elif var == 'MINERAL-AREA':
                mineral_locaiton[objects[0]][2] = v
            elif var == 'MINERAL-VALUE':
                mineral_locaiton[objects[0]][3] = v

        return {'mineral_location': mineral_locaiton}

    def build_states_layout(self, state):
        rover_location = {o: [None, None] for o in self._objects['rover']}
        mineral_harvested = {o: None for o in self._objects['mineral']}

        for k, v in state.items():
            var, objects = self._model.parse(k)
            if var == 'pos-x':
                rover_location[objects[0]][0] = v
            elif var == 'pos-y':
                rover_location[objects[0]][1] = v

        for k, v in state.items():
            var, objects = self._model.parse(k)
            if var == 'mineral-harvested':
                mineral_harvested[objects[0]] = v

        return {'mineral_harvested': mineral_harvested,
                'rover_location': rover_location}

    def init_canvas(self, figure_size, dpi):
        fig = plt.figure(figsize=figure_size, dpi=dpi)
        ax = plt.gca()
        plt.xlim([-figure_size[0] // 2, figure_size[0] // 2])
        plt.ylim([-figure_size[1] // 2, figure_size[1] // 2])
        plt.axis('scaled')
        plt.axis('off')
        return fig, ax

    def convert2img(self, fig, ax):
        ax.set_position((0, 0, 1, 1))
        fig.canvas.draw()

        data = np.frombuffer(fig.canvas.tostring_rgb(), dtype=np.uint8)
        data = data.reshape(fig.canvas.get_width_height()[::-1] + (3,))

        img = Image.fromarray(data)

        self._data = data
        self._img = img

        return img

    def render(self, state):
        self.states = state
        self._fig, self._ax = self.init_canvas(self._figure_size, self._dpi)

        nonfluent_layout = self.build_nonfluents_layout()
        state_layout = self.build_states_layout(state)

        self._nonfluent_layout = nonfluent_layout
        self._state_layout = state_layout

        max_value = max([v[3]
                         for k, v in nonfluent_layout['mineral_location'].items()])
        for k, v in nonfluent_layout['mineral_location'].items():
            value = nonfluent_layout['mineral_location'][k][3] / max_value
            if state_layout['mineral_harvested'][k] == False:
                p_point = plt.Circle((v[0], v[1]),
                                     radius=v[2], ec='forestgreen', fc='g',
                                     fill=True, linewidth=10,
                                     alpha=max(min(value, 0.9), 0.1))
            else:
                p_point = plt.Circle((v[0], v[1]),
                                     radius=v[2], ec='forestgreen', linewidth=10,
                                     fill=False, alpha=max(min(value, 0.9), 0.1))
            plt.text(v[0] - 1,
                     v[1],
                     "Value: %s" % self._nonfluent_layout['mineral_location'][k][3],
                     color='black', fontsize=50)
            self._ax.add_patch(p_point)

        for k, v in state_layout['rover_location'].items():
            rover_rec = plt.Rectangle((v[0], v[1]),
                                      0.5,
                                      0.5,
                                      fc='grey', zorder=2)
            self._ax.add_patch(rover_rec)

        img = self.convert2img(self._fig, self._ax)

        self._ax.cla()
        plt.cla()
        plt.close()

        return img

    def gen_inter_state(self, beg_state, end_state, steps):
        state_buffer = []
        beg_x = beg_state['xPos']
        beg_y = beg_state['yPos']
        end_x = end_state['xPos']
        end_y = end_state['yPos']

        if (beg_x, beg_y) == (end_x, end_y):
            state_buffer = [beg_state, end_state]
        else:
            for i in range(steps):
                x = beg_x + 1 / steps * (end_x - beg_x) * i
                y = beg_y + 1 / steps * (end_y - beg_y) * i
                state = beg_state.copy()
                state['xPos'] = x
                state['yPos'] = y
                state_buffer.append(state)
            state_buffer.append(end_state)

        return state_buffer



info = {
    'name': 'SysAdmin_POMDP',
    'description': 'An example RDDL description for the well-known SysAdmin problem. This is the pomdp version',
    'context': 'ippc2011',
    'tags': '',
    'viz': 'None'
}


import os.path
import sys
import importlib.util
from typing import List

from .ErrorHandling import RDDLRepoInstanceNotExist, RDDLRepoUnresolvedDependency


sys.path.append(os.path.join('..', 'rddlrepository'))
viz_backend_package_name = 'pyRDDLGym'


class ProblemInfo:
    def __init__(self, problem_data: dict) -> None:
        self.name = problem_data['name']
        self.desc = problem_data['description']
        self.loc = problem_data['location']
        self.instances = problem_data['instances']
        self.viz = problem_data['viz']

    def get_domain(self) -> str:
        path = os.path.join(self.loc, 'domain.rddl')
        return path

    def get_instance(self, num: int) -> str:
        if str(num) not in self.instances:
            raise RDDLRepoInstanceNotExist('problem ' + self.name + ' does not have instance ' + str(num))
        instance = 'instance' + str(num) + '.rddl'
        path = os.path.join(self.loc, instance)
        return path

    def list_instances(self, verbose=False) -> List[str]:
        if verbose:
            print(self.instances)
        return self.instances

    def get_visualizer(self):
        if self.viz == 'None':
            return None

        spec = importlib.util.find_spec(viz_backend_package_name)
        if spec is None:
            raise RDDLRepoUnresolvedDependency(viz_backend_package_name + " is not installed")

        path_to_viz = []
        p = os.path.split(self.loc)
        while p[1] != 'rddlrepository':
            path_to_viz.insert(0, p[1])
            p = os.path.split(p[0])
        path_to_viz.insert(0, p[1])
        path_to_viz = '.'.join(path_to_viz)

        viz = None
        viz_info = self.viz
        if viz_info:
            module, viz_class_name = viz_info.strip().split('.')
            viz_package_name = path_to_viz + '.' + module
            viz_package = __import__(viz_package_name, {}, {}, viz_class_name)
            viz = getattr(viz_package, viz_class_name)
        return viz



info = {
    'name': 'MountainCar',
    'description': 'A simple continuous MDP for the classical mountain car control problem.',
    'context': '',
    'tags': '',
    'viz': 'MountainCarViz.MountainCarVisualizer'
}


info = {
    'name': 'GameOfLife_POMDP',
    'description': 'A simple DBN to encode Conway\'s cellular automata game of life on a grid. This is the pomdp version.',
    'context': 'ippc2011',
    'tags': '',
    'viz': 'None'
}


class RDDLRepoEnvironmentNotExist(ValueError):
    pass

class RDDLRepoInstanceNotExist(ValueError):
    pass

class RDDLRepoDomainNotExist(ValueError):
    pass

class RDDLRepoProblemDuplication(ValueError):
    pass

class RDDLRepoUnresolvedDependency(ValueError):
    pass

class RDDLRepoManifestEmpty(ValueError):
    pass

class RDDLRepoContextNotExist(ValueError):
    pass



info = {
    'name': 'AcademicAdvising_POMDP',
    'description': 'In this domain, a student may take courses at a given cost and passes the course with a probability determined by how many of the prerequisites they have successfully passed. This is the pomdp version',
    'context': 'ippc2014',
    'tags': '',
    'viz': 'None'
}


from Manager.RDDLRepoManager import RDDLRepoManager as RDDLRepoManager


def main():

    # initializer repo manager, repo manifest will be loaded or built automatically if not present.
    # rebuild argument force manifest rebuild.
    # manager = RDDLRepoManager()
    manager = RDDLRepoManager(rebuild=True)

    # list all problems in the repo, unique name is generated based on the name and context of the problem
    manager.list_problems()

    manager.list_context()

    manager.list_problems_by_context('ippc2018')

    # getting a specific problem from the repo.
    # envInfo = manager.get_problem('RaceCar')
    envInfo = manager.get_problem('EarthObservation_ippc2018')

    # list available instances in for the problem.
    envInfo.list_instances()

    # return path to the domain file to be used/open by external tools.
    envInfo.get_domain()

    # return path to the desired instance file to be used/open by external tools.
    envInfo.get_instance(1)

    # return the pyRDDLGym visualizer object (pyRDDLGym dependency required) if exists, otherwise return None.
    envInfo.get_visualizer()





if __name__ == '__main__':
    main()



import matplotlib.pyplot as plt
import numpy as np
from PIL import Image

from pyRDDLGym.Core.Compiler.RDDLModel import PlanningModel
from pyRDDLGym import Visualizer
from pyRDDLGym.Visualizer.StateViz import StateViz


class PowerGenVisualizer(StateViz):

    def __init__(self, model: PlanningModel, dpi=20, fontsize=8, display=False) -> None:

        self._model = model
        self._states = model.states
        self._nonfluents = model.groundnonfluents()
        self._objects = model.objects
        self._figure_size = None
        self._dpi = dpi
        self._fontsize = fontsize
        self._interval = 15

        self._asset_path = "/".join(Visualizer.__file__.split("/")[:-1])

        self._nonfluent_layout = None
        self._state_layout = None
        self._fig, self._ax = None, None
        self._data = None
        self._img = None
    
    def build_nonfluents_layout(self): 

        prod_units_min = {o: None for o in self._objects['plant']}
        prod_units_max = {o: None for o in self._objects['plant']}
        prod_change_penalty = {o: None for o in self._objects['plant']}
        cost_per_unit = {o: None for o in self._objects['plant']}

        # add none-fluents
        for k, v in self._nonfluents.items():
            if 'PROD-UNITS-MIN_' in k:
                point = k.split('_')[1]
                prod_units_min[point] = v
            elif 'ROD-UNITS-MAX_' in k:
                point = k.split('_')[1]
                prod_units_max[point] = v
            elif 'PROD-CHANGE-PENALTY_' in k:
                point = k.split('_')[1]
                prod_change_penalty[point] = v
            elif 'COST-PER-UNIT_' in k:
                point = k.split('_')[1]
                cost_per_unit[point] = v

        return {'prod_units_min': prod_units_min,
                'prod_units_max': prod_units_max,
                'prod_change_penalty': prod_change_penalty,
                'cost_per_unit': cost_per_unit}

    def build_states_layout(self, state):
        prevProd = {o: None for o in self._objects['plant']}
        temperature = None

        for k, v in state.items():
            if 'prevProd_' in k:
                point = k.split('_')[1]
                prevProd[point] = v
            elif 'temperature' in k:
                temperature = v
        
        return {'prevProd': prevProd, 'temperature': temperature}

    def init_canvas_info(self):
        interval = self._interval
        plant_list = self._objects['plant']
        room_grid_size = (int((len(plant_list)) ** 0.5) + 1,
                          int((len(plant_list)) ** 0.5) + 1)
        room_center = (room_grid_size[0] * self._interval / 2,
                       room_grid_size[1] * self._interval / 2)
        start_points_zone = []
        
        for i in range(room_grid_size[0]):
            for j in range(room_grid_size[1]):
                start_points_zone.append((i * 15, j * 15))

        start_points_zone.sort(key=lambda x: x[1])
        start_points_zone.sort(key=lambda x: x[0], reverse=True)

        init_points = {}
        for i in range(len(plant_list)):
            init_points[plant_list[i]] = (
                start_points_zone[i][0], start_points_zone[i][1])

        canvas_info = {'canvas_size': (room_grid_size[0] * interval,
                                       room_grid_size[1] * interval),
                       'plant_init_points': init_points}

        return canvas_info
        
    def init_canvas(self, figure_size, dpi):
        fig = plt.figure(figsize=figure_size, dpi=dpi)
        ax = plt.gca()
        plt.xlim([-2, figure_size[0]])
        plt.ylim([-2, figure_size[1]])
        plt.axis('scaled')
        plt.axis('off')
        return fig, ax

    def render_plant(self, plant, nonfluent_layout, state_layout):

        fig = self._fig
        ax = self._ax
        interval = self._interval * 2 / 3
        init_x, init_y = self._canvas_info['plant_init_points'][plant]

        prod_min = nonfluent_layout['prod_units_min'][plant]
        prod_max = nonfluent_layout['prod_units_max'][plant]

        change_penalty = nonfluent_layout['prod_change_penalty'][plant]
        unit_cost = nonfluent_layout['cost_per_unit'][plant]

        prev_prod = state_layout['prevProd'][plant]
        temp = state_layout['temperature']

        upL = [init_x, init_y + interval * 1.2]
        upR = [init_x + interval, init_y + interval * 1.2]
        maxL = [init_x, interval + init_y]
        maxR = [init_x + interval, interval + init_y]
        lowL = [init_x, prod_min / prod_max * interval + init_y]
        lowR = [init_x + interval, prod_min / prod_max * interval + init_y]

        set_lw = 5

        line_max = plt.Line2D((maxL[0], maxR[0]),
                              (maxL[1], maxR[1]),
                              ls='--', color='orange', lw=set_lw, zorder=3)
        line_up = plt.Line2D((upL[0], upR[0]),
                             (upL[1], upR[1]),
                             ls='--', color='black', lw=set_lw, zorder=3)
        line_low = plt.Line2D((lowL[0], lowR[0]),
                              (lowL[1], lowR[1]),
                              ls='--', color='orange', lw=set_lw, zorder=3)
        lineL = plt.Line2D((init_x, upL[0]),
                           (init_y, upL[1]),
                           color='black', lw=set_lw, zorder=3)
        lineR = plt.Line2D((init_x + interval, upR[0]),
                           (init_y, upR[1]),
                           color='black', lw=set_lw, zorder=3)
        lineB = plt.Line2D((init_x, init_x + interval),
                           (init_y, init_y),
                           color='black', lw=set_lw, zorder=3)

        ax.add_line(line_max)
        ax.add_line(line_up)
        ax.add_line(line_low)
        ax.add_line(lineL)
        ax.add_line(lineR)
        ax.add_line(lineB)
        
        prod_rect = plt.Rectangle((init_x, init_y),
                                  interval,
                                  prev_prod / prod_max * interval,
                                  fc='limegreen',
                                  zorder=2)
        temp_rect = plt.Rectangle((init_x, init_y),
                                  interval,
                                  interval * 1.2,
                                  fc='firebrick',
                                  alpha=max(min(temp, 100), 0) / 100,
                                  zorder=2)
        cost_rect = plt.Rectangle((init_x, upL[1]),
                                  interval / 2,
                                  interval / 5,
                                  fc='deepskyblue',
                                  alpha=max(min(unit_cost, 10), 0) / 10,
                                  zorder=2)
        pen_rect = plt.Rectangle((init_x + interval / 2, upL[1]),
                                 interval / 2,
                                 interval / 5,
                                 fc='darkcyan',
                                 alpha=max(min(change_penalty, 10), 0) / 10,
                                 zorder=2)

        ax.add_patch(prod_rect)
        ax.add_patch(temp_rect)
        ax.add_patch(cost_rect)
        ax.add_patch(pen_rect)

        plt.text(init_x + interval * 1.1,
                 init_y + interval * 1.3,
                 "%s" % plant,
                 color='black', fontsize=50)

        return fig, ax

    def convert2img(self, fig, ax):
        
        ax.set_position((0, 0, 1, 1))
        fig.canvas.draw()

        data = np.frombuffer(fig.canvas.tostring_rgb(), dtype=np.uint8)
        data = data.reshape(fig.canvas.get_width_height()[::-1] + (3,))

        img = Image.fromarray(data)

        self._data = data
        self._img = img

        return img

    def fig2npa(self, fig):
        data = np.frombuffer(fig.canvas.tostring_rgb(), dtype=np.uint8)
        data = data.reshape(fig.canvas.get_width_height()[::-1] + (3,))
        return data

    def render(self, state):
        self.states = state

        self._nonfluent_layout = self.build_nonfluents_layout()
        self._state_layout = self.build_states_layout(state)
        self._canvas_info = self.init_canvas_info()
        self._figure_size = self._canvas_info['canvas_size']
        self._fig, self._ax = self.init_canvas(self._figure_size, self._dpi)

        for plant in self._objects['plant']: 
            self.render_plant(plant, self._nonfluent_layout, self._state_layout)

        img = self.convert2img(self._fig, self._ax)
            
        self._ax.cla()
        plt.cla()
        plt.close()

        return img




info = {
    'name': 'Elevators_POMDP',
    'description': 'This domain has a number of elevators delivering passengers to either the top or the bottom floor. This is the pomdp version.',
    'context': 'ippc2011',
    'tags': '',
    'viz': 'None'
}


import math
import numpy as np
from PIL import Image
import pygame
from pygame import gfxdraw

from pyRDDLGym.Core.Compiler.RDDLModel import PlanningModel
from pyRDDLGym.Visualizer.StateViz import StateViz


# code comes from openai gym
class MountainCarVisualizer(StateViz):

    def __init__(self, model: PlanningModel, figure_size=[600, 400], wait_time=100) -> None:
        self._model = model
        self._figure_size = figure_size
        self._wait_time = wait_time
        
        self._nonfluents = model.nonfluents
    
    def init_canvas(self, figure_size):
        screen = pygame.Surface(figure_size)
        surf = pygame.Surface(figure_size)
        return screen, surf
        
    def convert2img(self, screen):
        data = np.transpose(np.array(pygame.surfarray.pixels3d(screen)), axes=(1, 0, 2))
        img = Image.fromarray(data)
        return img
    
    def render(self, state):
        screen, surf = self.init_canvas(self._figure_size)
        
        world_width = self._nonfluents['MAX-POS'] - self._nonfluents['MIN-POS']
        scale = self._figure_size[0] / world_width
        carwidth = 40
        carheight = 20
        
        surf.fill((255, 255, 255))
        xs = np.linspace(self._nonfluents['MIN-POS'],
                         self._nonfluents['MAX-POS'], 100)
        ys = np.sin(3 * xs) * self._nonfluents['DEPTH'] + \
            (1. - self._nonfluents['DEPTH'])
        xys = list(zip((xs - self._nonfluents['MIN-POS']) * scale, ys * scale))
        pygame.draw.aalines(surf, points=xys, closed=False, color=(0, 0, 0))
        
        clearance = 10
        l, r, t, b = -carwidth / 2, carwidth / 2, carheight, 0
        coords = []
        for c in [(l, b), (l, t), (r, t), (r, b)]:
            c = pygame.math.Vector2(c).rotate_rad(math.cos(3 * state['pos']))
            coords.append(
                (
                    c[0] + (state['pos'] - self._nonfluents['MIN-POS']) * scale,
                    c[1] + clearance + (
                        np.sin(3 * state['pos']) * self._nonfluents['DEPTH'] + \
                        (1. - self._nonfluents['DEPTH'])
                    ) * scale,
                )
            )

        gfxdraw.aapolygon(surf, coords, (0, 0, 0))
        gfxdraw.filled_polygon(surf, coords, (0, 0, 0))
        
        for c in [(carwidth / 4, 0), (-carwidth / 4, 0)]:
            c = pygame.math.Vector2(c).rotate_rad(math.cos(3 * state['pos']))
            wheel = (
                int(c[0] + (state['pos'] - self._nonfluents['MIN-POS']) * scale),
                int(c[1] + clearance + (
                    np.sin(3 * state['pos']) * self._nonfluents['DEPTH'] + \
                    (1. - self._nonfluents['DEPTH'])
                ) * scale),
            )
            gfxdraw.aacircle(
                surf, wheel[0], wheel[1], int(carheight / 2.5), (128, 128, 128)
            )
            gfxdraw.filled_circle(
                surf, wheel[0], wheel[1], int(carheight / 2.5), (128, 128, 128)
            )
            
        flagx = int((self._nonfluents['GOAL-MIN'] - self._nonfluents['MIN-POS']) * scale)
        flagy1 = int((np.sin(3 * self._nonfluents['GOAL-MIN']) * self._nonfluents['DEPTH'] + \
                      (1. - self._nonfluents['DEPTH'])) * scale)
        flagy2 = flagy1 + 50
        gfxdraw.vline(surf, flagx, flagy1, flagy2, (0, 0, 0))
        
        gfxdraw.aapolygon(
            surf,
            [(flagx, flagy2), (flagx, flagy2 - 10), (flagx + 25, flagy2 - 5)],
            (204, 204, 0),
        )
        gfxdraw.filled_polygon(
            surf,
            [(flagx, flagy2), (flagx, flagy2 - 10), (flagx + 25, flagy2 - 5)],
            (204, 204, 0),
        )
        
        surf = pygame.transform.flip(surf, False, True)
        screen.blit(surf, (0, 0))

        pygame.time.wait(self._wait_time)
        
        img = self.convert2img(screen)
        
        del screen, surf
        
        return img
    



info = {
    'name': 'EarthObservation',
    'description': 'The Earth Observation domain models a satellite orbiting Earth that can take pictures of the landscape below with a camera.',
    'context': 'ippc2018',
    'tags': '',
    'viz': 'None'
}



info = {
    'name': 'UAV_Discrete',
    'description': 'Discrete action space version of multi-UAV problem where a group of UAVs have to reach goal positions in the 3d Space.',
    'context': '',
    'tags': '',
    'viz': 'UAVsViz.UAVsVisualizer'
}


info = {
    'name': 'Traffic_POMDP',
    'description': 'A simple binary version of the cell transition model (CTM) for modeling traffic. This is the pomdp verion',
    'context': 'ippc2014',
    'tags': '',
    'viz': 'None'
}



info = {
    'name': 'CooperativeRecon_MDP',
    'description': 'There is a 2d grid with an agent, a base, some hazard squares, and objects in different locations.',
    'context': 'ippc2011',
    'tags': '',
    'viz': 'None'
}


import copy
import os
import importlib
import csv
from typing import List

from .ErrorHandling import RDDLRepoDomainNotExist, RDDLRepoProblemDuplication, RDDLRepoManifestEmpty,RDDLRepoContextNotExist
from .ProblemInfo import ProblemInfo

HEADER = ['name', 'description', 'location', 'instances', 'viz', 'context', 'tags']
manifest = 'manifest.csv'


class RDDLRepoManager:
    def __init__(self, rebuild=False) -> None:
        self.archiver_dict= {}
        self.archive_by_context = {}
        self.manager_path = os.path.dirname(os.path.abspath(__file__))
        manifest_path = os.path.join(self.manager_path, manifest)
        if os.path.isfile(manifest_path) and not rebuild:
            try:
                self._load_repo()    # load repo to dict
            except:
                raise RDDLRepoManifestEmpty('An error ocurred while loading current repo manifest, please try to re-run with rebuild=True')
        else:
            self._build_repo()   # build repo and load to dict

    def list_problems(self, verbose=False) -> List[str]:
        problem_list = []
        if len(self.archiver_dict) == 0:
            raise RDDLRepoManifestEmpty('Repo manifest is empty please re-run with rebuild=True')
        for key, values in self.archiver_dict.items():
            if verbose:
                print(key + ": " + values['description'])
            problem_list.append(key)
        return problem_list

    def list_context(self, verbose=False) -> List[str]:
        context_list = []
        if len(self.archive_by_context) == 0:
            raise RDDLRepoManifestEmpty('Repo manifest is empty please re-run with rebuild=True')
        for key, _ in self.archive_by_context.items():
            if verbose:
                print(key)
            context_list.append(key)
        return context_list

    def list_problems_by_context(self, context: str, verbose=False) -> List[str]:
        if context not in self.archive_by_context:
            raise RDDLRepoContextNotExist('context: ' + context + ' does not exist in the RDDL repo')
        problems = '\n'.join(self.archive_by_context[context])
        if verbose:
            print(problems)
        problems_list = copy.deepcopy(self.archive_by_context[context])
        return problems_list

    def get_problem(self, name: str) -> ProblemInfo:
        if name in self.archiver_dict.keys():
            return ProblemInfo(self.archiver_dict[name])
        else:
            raise RDDLRepoDomainNotExist('Domain: ' + name + ' does not exists in the repository')

    def _build_repo(self) -> None:
        root_path = os.path.dirname(os.path.abspath(__file__))
        path_to_manifest = os.path.join(root_path, manifest)
        root_path = os.path.split(root_path)[0]
        archive_dir = os.path.join(root_path,'Archive')
        start_char = len(archive_dir)

        # build the repo dictionary as first step to verify correctness and uniqueness
        for root, dirs, files in os.walk(archive_dir, topdown=False):
            dir = root[start_char:]
            if '__pycache__' in dirs:
                dirs.remove('__pycache__')
            if len(dirs) > 0:
                continue
            d = os.path.split(root_path)
            if d[1] == '__pycache__':
                continue
            if "__init__.py" in files:
                d = self.__split_path_to_list(dir)
                module = 'rddlrepository.Archive' + '.' + '.'.join(d)
                mymodule = importlib.import_module(module)
                context = mymodule.info['context']
                if context:
                    context = '_' + context
                name = mymodule.info['name'] + context
                if name in self.archiver_dict.keys():
                    raise RDDLRepoProblemDuplication('domain: ' + name + ' already exists, problem names must be unique')
                if 'domain.rddl' not in files:
                    raise RDDLRepoDomainNotExist('domain: ' + name + ' does not have a domain.rddl file')
                instances = [fname[8:-5] for fname in files
                             if fname.startswith('instance') and fname.endswith('.rddl')]
                instances.sort(key=lambda x: int(x))
                self.archiver_dict[name] = {'name': name,
                                                        'description': mymodule.info['description'],
                                                        'location': root,
                                                        'instances': instances,
                                                        'viz': mymodule.info['viz'],
                                                        'context': mymodule.info['context'],
                                                        'tags': mymodule.info['tags']
                                            }

                context = mymodule.info['context']
                if context == '':
                    context = 'independent'
                if context in self.archive_by_context:
                    self.archive_by_context[context].append(name)
                else:
                    self.archive_by_context[context] = [name]


        # Generate manifest
        with open(path_to_manifest, 'w', newline='') as file:

            # write the csv header of the manifest
            writer = csv.writer(file, delimiter=',')
            writer.writerow(HEADER)

            # iterate through the dictionary
            for keys, values in self.archiver_dict.items():
                values_copy = copy.deepcopy(values)
                values_copy['instances'] = ','.join(values['instances'])
                values_copy['tags'] = ','.join(values['tags'])
                row = [values_copy[key] for key in HEADER]
                writer.writerow(row)

    def _load_repo(self) -> dict:
        root_path = os.path.dirname(os.path.abspath(__file__))
        path_to_manifest = os.path.join(root_path, manifest)
        if not os.path.isfile(path_to_manifest):
            return {}

        self.archiver_dict= {}
        with open(path_to_manifest) as file:
            reader = csv.reader(file, delimiter=',')
            for i, row in enumerate(reader):
                if i > 0:
                    name, *entries = row
                    self.archiver_dict[name] = dict(zip(HEADER[1:], entries))
                    self.archiver_dict[name]['name'] = name
                    self.archiver_dict[name]['instances'] = (self.archiver_dict[name]['instances']).split(',')
                    if entries[4] == '':
                        context = 'independent'
                    else:
                        context = entries[4]
                    if context in self.archive_by_context:
                        self.archive_by_context[context].append(name)
                    else:
                        self.archive_by_context[context] = [name]
            return self.archiver_dict

    def __split_path_to_list(self, path):
        l = []
        a = os.path.split(path)
        while a[1] != '':
            l.insert(0, a[1])
            a = os.path.split(a[0])
        return l




info = {
    'name': 'ChromaticDice',
    'description': 'Chromatic Dice is a variant of the popular dice game Yahtzee (also known as Kniffel).',
    'context': 'ippc2018',
    'tags': '',
    'viz': 'None'
}



# This file is part of rddlrepository

# rddlrepository is free software: you can redistribute it and/or modify
# it under the terms of the MIT License as published by
# the Free Software Foundation.

# rddlrepository is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# MIT License for more details.

# You should have received a copy of the MIT License
# along with rddlrepository. If not, see <https://opensource.org/licenses/MIT>.

from setuptools import setup, find_packages


setup(
      name='rddlrepository',
      version='0.10',
      author="Ayal Taitler, Michael Gimelfarb, Scott Sanner",
      author_email="ataitler@gmail.com, mike.gimelfarb@mail.utoronto.ca, ssanner@mie.utoronto.ca",
      description="Home for all things RDDL",
      license="MIT License",
      url="https://github.com/ataitler/rddlrepository",
      packages=find_packages(),
      install_requires=['numpy'],
      python_requires=">=3.8",
      include_package_data=True,
      classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
)




import matplotlib.pyplot as plt
import numpy as np
from PIL import Image

from pyRDDLGym.Core.Compiler.RDDLModel import PlanningModel
import pyRDDLGym.Visualizer as Visualizer
from pyRDDLGym.Visualizer.StateViz import StateViz


class RecSimVisualizer(StateViz):

    def __init__(self, model: PlanningModel,
                 figure_size=[5, 10],
                 dpi=100,
                 fontsize=10,
                 display=False) -> None:
        self._model = model
        self._states = model.states
        self._nonfluents = model.groundnonfluents()
        self._objects = model.objects
        self._figure_size = figure_size
        self._display = display
        self._dpi = dpi
        self._fontsize = fontsize
        self._interval = 10
        self._asset_path = "/".join(Visualizer.__file__.split("/")[:-1])
        self._object_layout = None
        self._fig, self._ax = None, None
        self._data = None
        self._img = None

        # if display == True:
        #     self.pygame_thread = threading.Thread(target=self.init_display)
        #     self.pygame_thread.start()
        _horizon = 10
        self._interp_steps = 25
        self._creator_disp = 64.0
        self._creator_fan_out = 2
        self._num_creator_clusters = 40
        self._num_topics = 2
        self._creator_boost_cap = 1.2
        self._iter = 0

    def build_nonfluents_layout(self):
        space_dim = len(self._model.objects['feature'])
        self._feature_index = {
            f: ind for ind, f in enumerate(self._model.objects['feature'])}
        if space_dim != 2:
            raise RuntimeError('This visualizer can only run in 2d (two features).')
        
        self._num_users = len(self._model.objects['consumer'])
        creators = self._model.objects['provider'].copy()
        creators.remove('pn')
        self._user_index = {
            c: ind for ind, c in enumerate(self._model.objects['consumer'])}
        self._num_creators = len(creators)
        self._creator_index = {p: ind for ind, p in enumerate(creators)}
        self._creator_means = np.zeros((self._num_creators, space_dim))
        self._user_interests = np.zeros((self._num_users, space_dim))

        for key, value in self._nonfluents.items():
            tokens = key.split('_')
            if tokens[0] == 'CONSUMER-AFFINITY':
                self._user_interests[
                    self._user_index[tokens[1]]][
                        self._feature_index[tokens[2]]] = value
            elif tokens[0] == 'PROVIDER-COMPETENCE':
                if tokens[1] == 'pn': continue
                self._creator_means[
                    self._creator_index[tokens[1]]][
                        self._feature_index[tokens[2]]] = value
        self._user_plot = None
        self._creator_plot = None
        self._creator_util_scatter = None
        return self._nonfluents
    
    def build_states_layout(self, states):
        self._user_satisfaction = np.zeros(self._num_users)
        self._creator_satisfaction = np.zeros(self._num_creators)
        for key, value in states.items():
            tokens = key.split('_')
            if tokens[0] == 'consumer-satisfaction':
                self._user_satisfaction[self._user_index[tokens[1]]] = value
            elif tokens[0] == 'provider-satisfaction':
                if tokens[1] == 'pn': continue
                self._creator_satisfaction[self._creator_index[tokens[1]]] = value
        return states
    
    def init_canvas(self, figure_size, dpi):
        width_box = 0.15
        rect_scatter = [0, 0, 1, 0.95]
        rect_creator_util = [0.62, 0.95, width_box, 0.05]
        rect_reset = [0.51, 0.95, width_box * 0.6, 0.05]
        rect_regenerate = [0.41, 0.95, width_box * 0.6, 0.05]
        rect_text = [0.01, 0.95, width_box, 0.05]
        rect_fair = [0.8, 0.95, width_box, 0.05]

        self._fig = plt.figure()
        self._ax_scatter = plt.axes(rect_scatter)
        self._ax_creator_util = plt.axes(rect_creator_util)
        self._ax_text = plt.axes(rect_text)
        self._axes = [self._ax_scatter, self._ax_creator_util, self._ax_text]

        return None, None

    def build_object_layout(self):
        return {'nonfluents_layout': self.build_nonfluents_layout(),
                'states_layout': self.build_states_layout()}

    def convert2img(self, fig, ax):
        
        # ax.set_position((0, 0, 1, 1))
        fig.canvas.draw()

        data = np.frombuffer(fig.canvas.tostring_rgb(), dtype=np.uint8)
        data = data.reshape(fig.canvas.get_width_height()[::-1] + (3,))

        img = Image.fromarray(data)

        self._data = data
        self._img = img

        return img

    def render(self, state):
        self.states = state

        self.init_canvas(self._figure_size, self._dpi)
        nonfluent_layout = self.build_nonfluents_layout()
        states_layout = self.build_states_layout(state)

        reward = self._user_satisfaction + 0.1
        rel_sizes = self._creator_satisfaction + 0.1
        # print(self._creator_satisfaction)
        rel_sizes = rel_sizes / np.sum(rel_sizes)
        # print(self._user_interests)
        if self._user_plot is None:
            self._user_plot = self._ax_scatter.scatter(
                self._user_interests[:, 0],
                self._user_interests[:, 1],
                s=reward,
                c=20 * np.sqrt(np.maximum(reward, 0.)),
                cmap='plasma',
                edgecolors='black')
        # print(self._creator_means[:, 0],
        #     self._creator_means[:, 1], rel_sizes)
        self._creator_plot = self._ax_scatter.scatter(
            self._creator_means[:, 0],
            self._creator_means[:, 1],
            s=200 * rel_sizes,
            c='red')
        self._ax_scatter.axis([
            np.min(self._user_interests[:, 0]) - 2,
            np.max(self._user_interests[:, 0]) + 2,
            np.min(self._user_interests[:, 1]) - 2,
            np.max(self._user_interests[:, 1]) + 2
        ])

        self._creator_util_scatter, = self._ax_creator_util.plot(
            range(self._num_creators), 20 * np.sort(rel_sizes), color='red')

        # plot design
        for ax in self._axes:
            ax.spines['right'].set_visible(False)
            ax.spines['top'].set_visible(False)
            ax.spines['bottom'].set_visible(False)
            ax.spines['left'].set_visible(False)
            ax.get_xaxis().set_visible(False)
            ax.get_yaxis().set_visible(False)

        # return [self._user_plot, self._creator_plot, self._creator_util_scatter]

        # state_layout = self.build_states_layout(state)
        # text_layout = {'state': state_layout}
        # text_str = pprint.pformat(text_layout)[1:-1]
        # self._ax.text(self._interval*0.5, self._figure_size[1]*self._interval*0.95, text_str, 
        #         horizontalalignment='left', verticalalignment='top', wrap=True, fontsize = self._fontsize)
        
        img = self.convert2img(self._fig, None)
        # img.save(f'frame{self._iter}.png')
        self._iter += 1

        # self._ax.cla()
        plt.close()
        # plt.show()

        return img
    



info = {
    'name': 'Navigation_MDP',
    'description': 'In a grid, a robot must get to a goal G, and every cell offers the robot a (different) chance of disappearing.',
    'context': 'ippc2011',
    'tags': '',
    'viz': 'None'
}